"""
test_request.py
====================================
Unit testing for request module
"""

import unittest
from imath_requests.request import PartData, ImageAnalysisData, ImageAnalysisFailure
from imath_requests.request import ImageValue, ImageMetaData
from imath_requests.request import Position, Dimension
from imath_requests.server import create_app
import requests
import multiprocessing


class TestPosition(unittest.TestCase):
    """
    Unit testing for Position class in requests module.

    """
    def test_init_position(self):
        position = Position(44.2, 17.4, 0.0)

    def test_position_json(self):
        position = Position(44.2, 17.4, 0.0)
        position_json = position.get_json()
        # TODO add position json generation test


class TestDimension(unittest.TestCase):
    """
    Unit testing for Dimension class in requests module.

    """
    def test_init_dimension(self):
        dimension = Dimension(44.2, 17.4, 0.0)

    def test_dimension_json(self):
        dimension = Dimension(5.2, 1.0, 0.0)
        dimension_json = dimension.get_json()
        # TODO add dimension json generation test


class TestRequestsPartData(unittest.TestCase):
    """
    Unit testing for PartData class in requests module.

    """
    def test_init_part_data(self):
        """
        Test generation of part data class
        """
        part_data_list = [
            {
                "key": "steel_grade",
                "value": "Grade01"
            },
            {
                "key": "heat_number",
                "value": "C1234566"
            },
            {
                "key": "rolling_schedule",
                "value": "Schedule1"
            },
            {
                "key": "analysis",
                "value": [
                    {
                        "key": "C",
                        "value": "0.2"
                    },
                    {
                        "key": "Mn",
                        "value": "0.02"
                    }
                ]
            }
        ]
        part_data = PartData(
            "1516193959559", "Part1234", "I3DR_DESKTOP_ABC123", part_data_list)

    def test_part_data_json(self):
        """
        Tests part data json generation.

        """
        part_data_list = [
            {
                "key": "steel_grade",
                "value": "Grade01"
            },
            {
                "key": "heat_number",
                "value": "C1234566"
            },
            {
                "key": "rolling_schedule",
                "value": "Schedule1"
            },
            {
                "key": "analysis",
                "value": [
                    {
                        "key": "C",
                        "value": "0.2"
                    },
                    {
                        "key": "Mn",
                        "value": "0.02"
                    }
                ]
            }
        ]
        part_data = PartData(
            "1516193959559", "Part1234", "I3DR_DESKTOP_ABC123", part_data_list)
        part_data.get_json()
        # TODO add part data json generation test


class TestRequestsImageValue(unittest.TestCase):
    """
    Unit testing for ImageValue class in requests module.

    """
    def test_init_image_value(self):
        """
        Test generation of image value class
        """
        image_value = ImageValue(
            "test001.png", "1516193959559", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), "1"
        )

    def test_image_value_json(self):
        """
        Tests image value json generation.

        """
        image_value = ImageValue(
            "test001.png", "1516193959559", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), "1"
        )
        image_value.get_json()
        # TODO add image value json generation test


class TestRequestsImageMetaData(unittest.TestCase):
    """
    Unit testing for ImageMetaData class in requests module.

    """
    def test_init_image_meta_data(self):
        """
        Test generation of image meta data class
        """
        image_value = ImageValue(
            "test001.png", "1516193959559", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), "1"
        )
        image_value_list = [
            image_value,
            image_value
        ]
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_meta_data = ImageMetaData(
            "Part1234", "Camera1", "I3DR_DESKTOP_ABC123",
            image_value_list, qualifying_metadata
        )

    def test_image_meta_data_json(self):
        """
        Tests image meta data json generation.

        """
        image_value = ImageValue(
            "test001.png", "1516193959559", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), "1"
        )
        image_value_list = [
            image_value,
            image_value
        ]
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_meta_data = ImageMetaData(
            "Part1234", "Camera1", "I3DR_DESKTOP_ABC123",
            image_value_list, qualifying_metadata
        )
        image_meta_data.get_json()
        # TODO add image meta data json generation test


class TestRequestsImageAnalysisFailure(unittest.TestCase):
    """
    Unit testing for ImageAnalysisFailure class in requests module.

    """
    def test_init_image_analysis_failure(self):
        """
        Test generation of image analysis failure class
        """
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_analysis_failure = ImageAnalysisFailure(
            "124355435321576", "4711", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), qualifying_metadata
        )

    def test_image_analysis_failure_json(self):
        """
        Tests image analysis failure json generation.

        """
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_analysis_failure = ImageAnalysisFailure(
            "124355435321576", "4711", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), qualifying_metadata
        )
        image_analysis_failure.get_json()
        # TODO add image analysis failure json generation test


class TestRequestsImageAnalysisData(unittest.TestCase):
    """
    Unit testing for ImageAnalysisData class in requests module.

    """
    def test_init_image_analysis_data(self):
        """
        Test generation of image analysis data class
        """
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_analysis_failure = ImageAnalysisFailure(
            "124355435321576", "4711", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), qualifying_metadata
        )
        image_analysis_failure_list = [
            image_analysis_failure,
            image_analysis_failure
        ]
        image_analysis_data = ImageAnalysisData(
            "Part1234", "I3DR_DESKTOP_ABC123",
            "test001.png", "1516193959559", image_analysis_failure_list
        )

    def test_image_analysis_data_json(self):
        """
        Tests image analysis data json generation.

        """
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_analysis_failure = ImageAnalysisFailure(
            "124355435321576", "4711", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), qualifying_metadata
        )
        image_analysis_failure_list = [
            image_analysis_failure,
            image_analysis_failure
        ]
        image_analysis_data = ImageAnalysisData(
            "Part1234", "I3DR_DESKTOP_ABC123",
            "test001.png", "1516193959559", image_analysis_failure_list
        )
        image_analysis_data.get_json()
        # TODO add image analysis data json generation test


class TestPartDataEndpoint(unittest.TestCase):
    """
    Unit testing part data in requests with mock server.

    """
    def setUp(self):
        self.server_name = "127.0.0.1:5000"
        self.app, self.api = create_app({'SERVER_NAME': self.server_name})
        self.app_thread = multiprocessing.Process(target=self.app.run)
        self.app_thread.start()
        # poll server to check it's ready for testing
        while True:
            try:
                requests.get("http://{}/".format(self.server_name), timeout=0.5)
                return
            except requests.exceptions.ConnectionError:
                pass

    def tearDown(self):
        self.app_thread.terminate()

    def test_part_data_get(self):
        url = "http://{}/api/image_analysis_data".format(self.server_name)
        image_analysis_data = ImageAnalysisData.get(url)
        self.assertIsNotNone(image_analysis_data)

    def test_part_data_post(self):
        url = "http://{}/api/image_analysis_data".format(self.server_name)
        qualifying_metadata = [
            {
                "key": "xxx",
                "value": "1"
            },
            {
                "key": "yyy",
                "value": "2"
            }
        ]
        image_analysis_failure = ImageAnalysisFailure(
            "124355435321576", "4711", Position(44.2, 17.4, 0.0),
            Dimension(5.2, 1.0, 0.0), qualifying_metadata
        )
        image_analysis_failure_list = [
            image_analysis_failure,
            image_analysis_failure
        ]
        image_analysis_data = ImageAnalysisData(
            "Part1234", "I3DR_DESKTOP_ABC123",
            "test001.png", "1516193959559", image_analysis_failure_list
        )
        resp = image_analysis_data.post(url)
        self.assertEqual(resp.status_code, 200)


if __name__ == '__main__':
    unittest.main()


"""
requests.py
====================================
Request handelling and data structures for interacting with iMath REST API
"""

import requests


class PartData:
    """
    Part data helper class.

    Includes addition functions creating json
    data from part data for easy use in REST API.

    Attributes
    ----------
    timestamp: str
        Timestamp e.g. 1516193959559
    part_id: str
        Unique part ID e.g. Part1234
    source: str
        Data source e.g. I3DR_DESKTOP_ABC123
    part_data: list
        Additional part data as a key value pair array

    """

    def __init__(
            self, timestamp: str, part_id: str,
            source: str, part_data: list):
        """
        Part data construction.

        Parameters
        ----------
        timestamp: str
            Timestamp e.g. 1516193959559
        part_id: str
            Unique part ID e.g. Part1234
        source: str
            Data source e.g. I3DR_DESKTOP_ABC123
        part_data: list
            Additional part data as a key value pair array

        """
        self.timestamp = timestamp
        self.part_id = part_id
        self.source = source
        self.part_data = part_data

    def get_json(self) -> dict:
        """
        Convert PartData into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        return {
            "timestamp": self.timestamp,
            "part_id": self.part_id,
            "source": self.source,
            "part_data": self.part_data
        }


class Dimension:
    """
    Dimension helper class.

    Includes addition functions creating json data
    from part data for easy use in REST API.

    Attributes
    ----------
    x: float
        X-axis dimension
    y: float
        Y-axis dimension
    z: float
        Z-axis dimension

    """
    def __init__(self, x:float, y:float, z: float):
        """
        Dimension construction.

        Parameters
        ----------
        x: float
            X-axis dimension
        y: float
            Y-axis dimension
        z: float
            Z-axis dimension

        """
        self.x = x
        self.y = y
        self.z = z

    def get_json(self):
        """
        Convert Dimension into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        return [
            self.x,
            self.y,
            self.z
        ]


class Position:
    """
    Position helper class.

    Includes addition functions creating json data
    from part data for easy use in REST API.

    Attributes
    ----------
    x: float
        X-axis position
    y: float
        Y-axis position
    z: float
        Z-axis position

    """
    def __init__(self, x:float, y:float, z: float):
        """
        Position construction.

        Parameters
        ----------
        x: float
            X-axis position
        y: float
            Y-axis position
        z: float
            Z-axis position

        """
        self.x = x
        self.y = y
        self.z = z

    def get_json(self):
        """
        Convert Position into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        return [
            self.x,
            self.y,
            self.z
        ]


class ImageValue:
    """
    Image Value helper class.

    Includes addition functions creating json data
    from part data for easy use in REST API.

    Attributes
    ----------
    value: str
        Name of the image e.g. test001.png
    timestamp: str
        Timestamp e.g. 1516193959559
    position: Position
        Position on the part where the image was taken
    dimension: Dimension
        Dimension of the area of the part, which is on the image
    quality: str
        Quality

    """

    def __init__(
            self, value: str, timestamp: str,
            position: Position, dimension: Dimension, quality: str):
        """
        Image value construction.

        Parameters
        ----------
        value: str
            Name of the image e.g. test001.png
        timestamp: str
            Timestamp e.g. 1516193959559
        position: str
            Position on the part where the image was taken
        dimension: str
            Dimension of the area of the part, which is on the image
        quality : str
            Quality

        """
        self.value = value
        self.timestamp = timestamp
        self.position = position
        self.dimension = dimension
        self.quality = quality

    def get_json(self) -> dict:
        """
        Convert ImageValue into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        return {
            "value": self.value,
            "timestamp": self.timestamp,
            "position": self.position.get_json(),
            "dimension": self.dimension.get_json(),
            "quality": self.quality
        }


class ImageMetaData:
    """
    Image meta data helper class.

    Includes addition functions creating json data
    from image meta data for easy use in REST API.

    Attributes
    ----------
    part_id: str
        Unique part ID e.g. Part1234
    value_id: str
        Unique identification of the capture device e.g. Camera1
    source: str
        Data source e.g. I3DR_DESKTOP_ABC123
    values: list
        List of ImageValues
    qualifying_metadata: list
        Qualifying metadata should be a key value pair array

    """

    def __init__(
            self, part_id: str, value_id: str,
            source: str, values: list, qualifying_metadata):
        """
        Image meta data construction.

        Parameters
        ----------
        part_id: str
            Unique part ID e.g. Part1234
        value_id: str
            Unique identification of the capture device e.g. Camera1
        source: str
            Data source e.g. I3DR_DESKTOP_ABC123
        values: list
            List of ImageValues
        qualifying_metadata: list
            Qualifying metadata should be a key value pair array

        """
        self.part_id = part_id
        self.value_id = value_id
        self.source = source
        self.values = values
        self.qualifying_metadata = qualifying_metadata

    def get_json(self) -> dict:
        """
        Convert ImageMetaData into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        values_json_list = []
        for value in self.values:
            values_json_list.append(value.get_json())
        return {
            "part_id": self.part_id,
            "value_id": self.value_id,
            "source": self.source,
            "values": values_json_list,
            "qualifying_metadata": self.qualifying_metadata
        }


class ImageAnalysisFailure:
    """
    Image analysis failure helper class.

    Includes addition functions creating json data
    from part data for easy use in REST API.

    Attributes
    ----------
    id: str
        Unique failure id e.g. 124355435321576
    failure: str
        Failure type id e.g. 4711
    position: Position
        Position on the part where the image was taken
    dimension: Dimension
        Dimension of the area of the part, which is on the image
    qualifying_metadata: list
        Qualifying metadata should be a key value pair array

    """
    def __init__(
            self, id: str, failure: str,
            position: Position, dimension: Dimension, qualifying_metadata: list):
        """
        Image analysis failure construction.

        Parameters
        ----------
        id: str
            Unique failure id e.g. 124355435321576
        failure: str
            Failure type id e.g. 4711
        position: Position
            Position on the part where the image was taken
        dimension: Dimension
            Dimension of the area of the part, which is on the image
        qualifying_metadata: list
            Qualifying metadata should be a key value pair array

        """
        self.id = id
        self.failure = failure
        self.position = position
        self.dimension = dimension
        self.qualifying_metadata = qualifying_metadata

    def get_json(self) -> dict:
        """
        Convert ImageAnalysisFailure into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        return {
            "id": self.id,
            "failure": self.failure,
            "position": self.position.get_json(),
            "dimension": self.dimension.get_json(),
            "qualifying_metadata": self.qualifying_metadata
        }

class ImageAnalysisData:
    """
    Image meta data helper class.

    Includes addition functions creating json data
    from image meta data for easy use in REST API.

    Attributes
    ----------
    part_id : str
        Unique part ID e.g. Part1234
    source : str
        Data source e.g. I3DR_DESKTOP_ABC123
    value: str
        List of ImageValues
    timestamp : str
        Timestamp e.g. 1516193959559
    failures: list
        list of ImageAnalysisFailure

    """

    def __init__(
            self, part_id: str, source: str,
            value: str, timestamp: str, failures: list):
        """
        Image Meta Data construction.

        Parameters
        ----------
        part_id : str
            Unique part ID e.g. Part1234
        source : str
            Data source e.g. I3DR_DESKTOP_ABC123
        value: str
            Name of the image (NOT ImageValue)
        timestamp : str
            Timestamp e.g. 1516193959559
        failures: list
            list of ImageAnalysisFailure

        """
        self.part_id = part_id
        self.source = source
        self.value = value 
        self.timestamp = timestamp
        self.failures = failures

    def get_json(self) -> dict:
        """
        Convert ImageAnalysisData into json string for use in REST API.

        Returns
        -------
        dict
            Json formatted string

        """
        failures_json_list = []
        for failure in self.failures:
            failures_json_list.append(failure.get_json())
        return {
        	"part_id": self.part_id,
        	"source": self.source,
        	"value": self.value,
        	"timestamp": self.timestamp,
        	"failures": failures_json_list
        }

    @staticmethod
    def from_json(json: str) -> 'ImageAnalysisData':
        part_id = json['part_id']
        source = json['source']
        value = json['value']
        timestamp = json['timestamp']
        failures = []
        for failure in json['failures']:
            position = Position(
                failure['position'][0],
                failure['position'][1],
                failure['position'][2])
            dimension = Dimension(
                failure['dimension'][0],
                failure['dimension'][1],
                failure['dimension'][2])
            failures.append(
                ImageAnalysisFailure(
                    failure['id'], failure['failure'], position,
                    dimension, failure['qualifying_metadata']
                )
            )
        return ImageAnalysisData(
            part_id, source, value, timestamp, failures)

    def post(self, url: str) -> None:
        data = self.get_json()
        response = requests.post(url, json = data)
        return response

    @staticmethod
    def get(url: str) -> 'ImageAnalysisData':
        json = requests.get(url).json()['data']
        return ImageAnalysisData.from_json(json)

if __name__ == "__main__":
    url = 'http://127.0.0.1:5000/api/image_analysis_data'
    # image_analysis_data = ImageAnalysisData.get(url)
    qualifying_metadata = [
        {
            "key": "xxx",
            "value": "1"
        },
        {
            "key": "yyy",
            "value": "2"
        }
    ]
    image_analysis_failure = ImageAnalysisFailure(
        "124355435321576", "4711", Position(44.2, 17.4, 0.0),
        Dimension(5.2, 1.0, 0.0), qualifying_metadata
    )
    image_analysis_failure_list = [
        image_analysis_failure,
        image_analysis_failure
    ]
    image_analysis_data = ImageAnalysisData(
        "Part1234", "I3DR_DESKTOP_ABC123",
        "test001.png", "1516193959559", image_analysis_failure_list
    )
    resp = image_analysis_data.post(url)
    print(resp)


"""
server.py
====================================
Mock server for testing REST API.
"""

import os
from flask import Flask, request, json, Response
from flask_restful import Resource, Api, reqparse


class PartDataEndpoint(Resource):
    """
    Part data API endpoint class.

    Resource used in Flask app to mock REST API.

    """
    def get(self):
        """
        REST API get request to return part data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        part_data = {
            "timestamp": 1516193959559,
            "part_id": "Part1234",
            "source": "Camera_Control_PC_Garret",
            "part_data": [
                {
                    "key": "steel_grade",
                    "value": "Grade01"
                },
                {
                    "key": "heat_number",
                    "value": "C1234566"
                },
                {
                    "key": "rolling_schedule",
                    "value": "Schedule1"
                },
                {
                    "key": "analysis",
                    "value": [
                        {
                            "key": "C",
                            "value": "0.2"
                        },
                        {
                            "key": "Mn",
                            "value": "0.02"
                        }
                    ]
                }
            ]
        }
        return {'data': part_data}, 200  # return data and 200 OK code

    def post(self):
        """
        REST API post request to accept part data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        json_data = request.json
        # TODO validate json data
        return {'data': json_data}, 200  # return data with 200 OK


class ImageMetaDataEndpoint(Resource):
    """
    Image meta data API endpoint class.

    Resource used in Flask app to mock REST API.

    """
    def get(self):
        """
        REST API get request to return image meta data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        image_meta_data = {
            "part_id" : "Part1234",
            "value_id" : "Camera1",
            "source": "Camera_Control_PC_Garret",
            "values" : [
                { 
                    "value": "Part1234_1.1_2.2_1.jpg",
                    "timestamp": "1516193959559",
                    "position": [346.2, 2.0, 0.0],
                    "dimension" : [5.2, 1.0, 0.0],
                    "quality" : "1"
                },
                { 
                    "value": "Part1234_1.1_2.2_2.jpg",
                    "timestamp": "1516193959559",
                    "position": [246.2, 5.0, 0.0],
                    "dimension" : [1.7, 4.0, 0.0],
                    "quality" : "-1"
                }
            ],
            "qualifying_metadata" : [
                { 
                    "key": "key1",
                    "value": "value1"
                },
                { 
                    "key": "key2",
                    "value": "value2"
                }
            ]
        }
        return {'data': image_meta_data}, 200  # return data and 200 OK code

    def post(self):
        """
        REST API post request to accept image meta data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        json_data = request.json
        # TODO validate json data
        return {'data': json_data}, 200  # return data with 200 OK


class ImageAnalysisDataEndpoint(Resource):
    """
    Image analysis data API endpoint class.

    Resource used in Flask app to mock REST API.

    """
    def get(self):
        """
        REST API get request to return image analysis data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        image_analysis_data = {
            "part_id": "Part1234",
            "source": "Analysis System",
            "value": "Part1234_1.1_2.2_1.jpg",
            "timestamp": "1516193959559",
            "failures": [
                {
                    "id": 124355435321576,
                    "failure": "4711",
                    "position": [
                        44.2,
                        17.4,
                        0.0
                    ],
                    "dimension": [
                        5.2,
                        1.0,
                        0.0
                    ],
                    "qualifying_metadata": [
                        {
                            "key": "xxx",
                            "value": "1"
                        },
                        {
                            "key": "yyy",
                            "value": "2"
                        }
                    ]
                },
                {
                    "id": 124355435321578,
                    "failure": "4712",
                    "position": [
                        33.2,
                        3.0,
                        0.0
                    ],
                    "dimension": [
                        2.3,
                        1.1,
                        0.0,
                    ],
                    "qualifying_metadata": [
                        {
                            "key": "xxx",
                            "value": "1"
                        },
                        {
                            "key": "yyy",
                            "value": "2"
                        }
                    ]
                }
            ]
        }
        return {'data': image_analysis_data}, 200  # return data and 200 OK code

    def post(self):
        """
        REST API post request to accept image analysis data.

        Returns
        -------
        tuple, int
            part data (json), status code

        """
        json_data = request.json
        print(json_data)
        # TODO validate json data
        return {'data': json_data}, 200  # return data with 200 OK


def create_app(test_config=None):
    """
    Create Flask app

    Create and configure the app to mockup server for iMath REST API.

    Parameters
    ----------
    test_config:
        Test config

    Returns
    -------
    app, api
        Flask App, Flask API

    """

    # create and configure the app
    app = Flask(__name__, instance_relative_config=True)
    app.config.from_mapping(
        SECRET_KEY='dev',
        DATABASE=os.path.join(app.instance_path, 'flaskr.sqlite'),
    )

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile('config.py', silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    api = Api(app)

    @app.route('/')
    def index():
        page = """ 
            <h1>Welcome to iMath Requests test server</h1>
            <h3>The following API endpoints are available:</h3>
            <ul>
                <li>/part_data</li>
                <li>/image_meta_data</li>
                <li>/image_analysis_data</li>
            </ul>
        """
        return page

    api.add_resource(PartDataEndpoint, '/api/part_data')
    api.add_resource(ImageMetaDataEndpoint, '/api/image_meta_data')
    api.add_resource(ImageAnalysisDataEndpoint, '/api/image_analysis_data')

    return app, api

if __name__ == "__main__":
    app, api = create_app()
    app.run()



"""
test_server.py
====================================
Unit testing for server module
"""

import unittest
import pytest
import tempfile
from imath_requests.server import create_app


class TestPartDataEndpoint(unittest.TestCase):
    """
    Unit testing for part data endpoint of app in Server module.

    """
    def setUp(self):
        app, api = create_app({'TESTING': True})
        self.client = app.test_client()

    def test_part_data_get(self):
        self.client.get('/api/part_data', follow_redirects=True)

    def test_part_data_post(self):
        part_data = {
            "timestamp": 1516193959559,
            "part_id": "Part1234",
            "source": "Camera_Control_PC_Garret",
            "part_data": [
                {
                    "key": "steel_grade",
                    "value": "Grade01"
                },
                {
                    "key": "heat_number",
                    "value": "C1234566"
                },
                {
                    "key": "rolling_schedule",
                    "value": "Schedule1"
                },
                {
                    "key": "analysis",
                    "value": [
                        {
                            "key": "C",
                            "value": "0.2"
                        },
                        {
                            "key": "Mn",
                            "value": "0.02"
                        }
                    ]
                }
            ]
        }
        self.client.post('/api/part_data', data=part_data, follow_redirects=True)
